@@ -14,7 +14,7 @@ int main(int argc, char *argv[]) {
 
     // Create entity, set value of position using reflection API
     ecs_entity_t ent = ecs_new_entity(ecs, "ent");
-    void *ptr = ecs_get_mut_id(ecs, ent, Position);
+    void *ptr = ecs_ensure_id(ecs, ent, Position);
 
     ecs_meta_cursor_t cur = ecs_meta_cursor(ecs, Position, ptr);
     ecs_meta_push(&cur);          // {
