@@ -9,6 +9,39 @@
 
 #ifdef FLECS_JSON
 
+typedef struct {
+    ecs_allocator_t *a;
+    ecs_vec_t records;
+    ecs_vec_t result_ids;
+    ecs_vec_t columns_set;
+    ecs_map_t anonymous_ids;
+    ecs_map_t missing_reflection;
+} ecs_from_json_ctx_t;
+
+static
+void flecs_from_json_ctx_init(
+    ecs_allocator_t *a,
+    ecs_from_json_ctx_t *ctx)
+{
+    ctx->a = a;
+    ecs_vec_init_t(a, &ctx->records, ecs_record_t*, 0);
+    ecs_vec_init_t(a, &ctx->result_ids, ecs_id_t, 0);
+    ecs_vec_init_t(a, &ctx->columns_set, ecs_id_t, 0);
+    ecs_map_init(&ctx->anonymous_ids, a);
+    ecs_map_init(&ctx->missing_reflection, a);
+}
+
+static
+void flecs_from_json_ctx_fini(
+    ecs_from_json_ctx_t *ctx)
+{
+    ecs_vec_fini_t(ctx->a, &ctx->records, ecs_record_t*);
+    ecs_vec_fini_t(ctx->a, &ctx->result_ids, ecs_record_t*);
+    ecs_vec_fini_t(ctx->a, &ctx->columns_set, ecs_id_t);
+    ecs_map_fini(&ctx->anonymous_ids);
+    ecs_map_fini(&ctx->missing_reflection);
+}
+
 static
 const char* flecs_json_parse_path(
     const ecs_world_t *world,
@@ -17,22 +50,31 @@ const char* flecs_json_parse_path(
     ecs_entity_t *out,
     const ecs_from_json_desc_t *desc)
 {
-    json = flecs_json_expect(json, JsonString, token, desc);
+    char *path = NULL;
+    json = flecs_json_expect_string(json, token, &path, desc);
     if (!json) {
         goto error;
     }
 
-    ecs_entity_t result = ecs_lookup_fullpath(world, token);
+    ecs_entity_t result = ecs_lookup(world, path);
     if (!result) {
         ecs_parser_error(desc->name, desc->expr, json - desc->expr, 
-            "unresolved identifier '%s'", token);
+            "unresolved identifier '%s'", path);
         goto error;
     }
 
     *out = result;
 
+    if (path != token) {
+        ecs_os_free(path);
+    }
+
     return json;
 error:
+    if (path != token) {
+        ecs_os_free(path);
+    }
+
     return NULL;
 }
 
@@ -216,12 +258,17 @@ const char* ecs_entity_from_json(
     }
 
     if (!ecs_os_strcmp(token, "path")) {
-        json = flecs_json_expect(json, JsonString, token, &desc);
+        char *path = NULL;
+        json = flecs_json_expect_string(json, token, &path, &desc);
         if (!json) {
             goto error;
         }
 
-        ecs_add_fullpath(world, e, token);
+        ecs_add_fullpath(world, e, path);
+
+        if (path != token) {
+            ecs_os_free(path);
+        }
 
         json = flecs_json_parse(json, &token_kind, token);
         if (!json) {
@@ -353,7 +400,7 @@ const char* ecs_entity_from_json(
         }
 
         /* Get mutable pointer */
-        void *comp_ptr = ecs_get_mut_id(world, e, id);
+        void *comp_ptr = ecs_ensure_id(world, e, id);
         if (!comp_ptr) {
             char *idstr = ecs_id_str(world, id);
             ecs_parser_error(name, expr, json - expr, 
@@ -443,6 +490,7 @@ static
 bool flecs_json_name_is_anonymous(
     const char *name)
 {
+    ecs_assert(name != NULL, ECS_INTERNAL_ERROR, NULL);
     if (isdigit(name[0])) {
         const char *ptr;
         for (ptr = name + 1; *ptr; ptr ++) {
@@ -484,11 +532,15 @@ ecs_entity_t flecs_json_ensure_entity(
              * to use. This allows the deserializer to bind to existing 
              * anonymous ids, as they will never be reissued. */
             deser_id = ecs_map_ensure(anonymous_ids, ser_id);
-            if (!ecs_exists(world, ser_id) || ecs_is_alive(world, ser_id)) {
+            if (!ecs_exists(world, ser_id) || 
+               (ecs_is_alive(world, ser_id) && !ecs_get_name(world, ser_id))) 
+            {
                 /* Only use existing id if it's alive or doesn't exist yet. The 
-                 * id could have been recycled for another entity */
+                 * id could have been recycled for another entity 
+                 * Also don't use existing id if the existing entity is not
+                 * anonymous. */
                 deser_id[0] = ser_id;
-                ecs_ensure(world, ser_id);
+                ecs_make_alive(world, ser_id);
             } else {
                 /* If id exists and is not alive, create a new id */
                 deser_id[0] = flecs_json_new_id(world, ser_id);
@@ -515,10 +567,12 @@ ecs_table_t* flecs_json_parse_table(
     ecs_world_t *world,
     const char *json,
     char *token,
+    ecs_from_json_ctx_t *ctx,
     const ecs_from_json_desc_t *desc)
 {
     ecs_json_token_t token_kind = 0;
-    ecs_table_t *table = NULL;
+
+    ecs_vec_clear(&ctx->result_ids);
 
     do {
         ecs_id_t id = 0;
@@ -527,24 +581,34 @@ ecs_table_t* flecs_json_parse_table(
             goto error;
         }
 
-        json = flecs_json_expect(json, JsonString, token, desc);
+        char *first_name = NULL;
+        json = flecs_json_expect_string(json, token, &first_name, desc);
         if (!json) {
             goto error;
         }
 
-        ecs_entity_t first = flecs_json_lookup(world, 0, token, desc);
+        ecs_entity_t first = flecs_json_lookup(world, 0, first_name, desc);
+        if (first_name != token) {
+            ecs_os_free(first_name);
+        }
+
         if (!first) {
             goto error;
         }
 
         json = flecs_json_parse(json, &token_kind, token);
         if (token_kind == JsonComma) {
-            json = flecs_json_expect(json, JsonString, token, desc);
+            char *second_name = NULL;
+            json = flecs_json_expect_string(json, token, &second_name, desc);
             if (!json) {
                 goto error;
             }
 
-            ecs_entity_t second = flecs_json_lookup(world, 0, token, desc);
+            ecs_entity_t second = flecs_json_lookup(world, 0, second_name, desc);
+            if (second_name != token) {
+                ecs_os_free(second_name);
+            }
+
             if (!second) {
                 goto error;
             }
@@ -563,10 +627,7 @@ ecs_table_t* flecs_json_parse_table(
             goto error;
         }
 
-        table = ecs_table_add_id(world, table, id);
-        if (!table) {
-            goto error;
-        }
+        ecs_vec_append_t(ctx->a, &ctx->result_ids, ecs_id_t)[0] = id;
 
         const char *lah = flecs_json_parse(json, &token_kind, token);
         if (token_kind == JsonComma) {
@@ -580,38 +641,87 @@ ecs_table_t* flecs_json_parse_table(
         }
     } while (json[0]);
 
+    /* Make a copy of the ids array because we need the original order for 
+     * deserializing the component values, and the sorted order for finding or
+     * creating the table. */
+    ecs_vec_t id_copy = ecs_vec_copy_t(ctx->a, &ctx->result_ids, ecs_id_t);
+    ecs_type_t type = { 
+        .array = ecs_vec_first(&id_copy),
+        .count = ecs_vec_count(&id_copy)
+    };
+
+    qsort(type.array, flecs_itosize(type.count), sizeof(ecs_id_t), 
+        flecs_id_qsort_cmp);
+
+    ecs_table_t *table = flecs_table_find_or_create(world, &type);
+    if (!table) {
+        goto error;
+    }
+
+    ecs_vec_fini_t(ctx->a, &id_copy, ecs_id_t);
+
     return table;
 error:
     return NULL;
 }
 
+static
+void flecs_json_zeromem_table(
+    ecs_table_t *table)
+{
+    int32_t count = ecs_vec_count(&table->data.entities);
+    int32_t i, column_count = table->column_count;
+    for (i = 0; i < column_count; i ++) {
+        ecs_column_t *column = &table->data.columns[i];
+        ecs_type_info_t *ti = column->ti;
+        if (!ti->hooks.ctor) {
+            void *ptr = ecs_vec_first(&column->data);
+            ecs_os_memset(ptr, 0, ti->size * count);
+        }
+    }
+}
+
 static
 int flecs_json_parse_entities(
     ecs_world_t *world,
-    ecs_allocator_t *a,
     ecs_table_t *table,
     ecs_entity_t parent,
     const char *json,
     char *token,
-    ecs_vec_t *records,
+    ecs_from_json_ctx_t *ctx,
     const ecs_from_json_desc_t *desc)
 {
     char name_token[ECS_MAX_TOKEN_SIZE];
     ecs_json_token_t token_kind = 0;
-    ecs_vec_clear(records);
+    ecs_vec_clear(&ctx->records);
 
     do {
+        char *name = NULL;
         json = flecs_json_parse(json, &token_kind, name_token);
         if (!json) {
             goto error;
         }
+
+        if (token_kind == JsonLargeString) {
+            ecs_strbuf_t large_token = ECS_STRBUF_INIT;
+            json = flecs_json_parse_large_string(json, &large_token);
+            if (!json) {
+                break;
+            }
+
+            name = ecs_strbuf_get(&large_token);
+            token_kind = JsonString;
+        } else {
+            name = name_token;
+        }
+
         if ((token_kind != JsonNumber) && (token_kind != JsonString)) {
             ecs_parser_error(desc->name, desc->expr, json - desc->expr, 
                 "expected number or string");
             goto error;
         }
 
-        ecs_entity_t e = flecs_json_lookup(world, parent, name_token, desc);
+        ecs_entity_t e = flecs_json_lookup(world, parent, name, desc);
         ecs_record_t *r = flecs_entities_try(world, e);
         ecs_assert(r != NULL, ECS_INTERNAL_ERROR, NULL);
 
@@ -621,13 +731,14 @@ int flecs_json_parse_entities(
                 ecs_commit(world, e, r, r->table, NULL, &r->table->type);
                 cleared = true;
             }
+
             ecs_commit(world, e, r, table, &table->type, NULL);
             if (cleared) {
-                char *entity_name = strrchr(name_token, '.');
+                char *entity_name = strrchr(name, '.');
                 if (entity_name) {
                     entity_name ++;
                 } else {
-                    entity_name = name_token;
+                    entity_name = name;
                 }
                 if (!flecs_json_name_is_anonymous(entity_name)) {
                     ecs_set_name(world, e, entity_name);
@@ -635,9 +746,20 @@ int flecs_json_parse_entities(
             }
         }
 
-        ecs_assert(table == r->table, ECS_INTERNAL_ERROR, NULL);
-        ecs_record_t** elem = ecs_vec_append_t(a, records, ecs_record_t*);
-        *elem = r;
+        if (name != name_token) {
+            ecs_os_free(name);
+        }
+
+        if (table != r->table) {
+            ecs_parser_error(desc->name, desc->expr, json - desc->expr, 
+                "invalid entity identifier");
+            goto error;
+        } else {
+            ecs_assert(table == r->table, ECS_INTERNAL_ERROR, NULL);
+            ecs_record_t** elem = ecs_vec_append_t(
+                ctx->a, &ctx->records, ecs_record_t*);
+            *elem = r;
+        }
 
         json = flecs_json_parse(json, &token_kind, token);
         if (token_kind == JsonArrayClose) {
@@ -654,46 +776,71 @@ error:
     return -1;
 }
 
+static
+const char* flecs_json_missing_reflection(
+    ecs_world_t *world,
+    ecs_id_t id,
+    const char *json,
+    char *token,
+    ecs_from_json_ctx_t *ctx,
+    const ecs_from_json_desc_t *desc)
+{
+    if (!desc->strict && ecs_map_get(&ctx->missing_reflection, id)) {
+        json = flecs_json_skip_array(json, token, desc);
+        goto done;
+    }
+
+    /* Don't spam log when multiple values of a type can't be deserialized */
+    ecs_map_ensure(&ctx->missing_reflection, id);
+
+    char *id_str = ecs_id_str(world, id);
+    ecs_parser_error(desc->name, desc->expr, json - desc->expr, 
+        "missing reflection for '%s'", id_str);
+    ecs_os_free(id_str);
+
+done:
+    if (desc->strict) {
+        return NULL;
+    } else {
+        return flecs_json_skip_array(json, token, desc);
+    }
+}
+
 static
 const char* flecs_json_parse_column(
     ecs_world_t *world,
     ecs_table_t *table,
-    int32_t index,
+    ecs_id_t id,
     const char *json,
     char *token,
-    ecs_vec_t *records,
+    ecs_from_json_ctx_t *ctx,
     const ecs_from_json_desc_t *desc)
 {
-    if (!table->column_count) {
-        ecs_parser_error(desc->name, desc->expr, json - desc->expr, 
-            "table has no components");
-        goto error;
+    /* If deserializing id caused trouble before, don't bother trying again */
+    if (!desc->strict && ecs_map_get(&ctx->missing_reflection, id)) {
+        return flecs_json_skip_array(json, token, desc);
     }
 
-    if (index >= table->type.count) {
-        ecs_parser_error(desc->name, desc->expr, json - desc->expr, 
-            "more value arrays than component columns in table");
-        goto error;
-    }
+    ecs_table_record_t *tr = flecs_table_record_get(world, table, id);
+    /* Table was created with this id, it should exist */
+    ecs_assert(tr != NULL, ECS_INTERNAL_ERROR, NULL);
 
-    int32_t data_column = table->column_map[index];
-    if (data_column == -1) {
-        char *table_str = ecs_table_str(world, table);
-        ecs_parser_error(desc->name, desc->expr, json - desc->expr, 
-            "values provided for tag at column %d of table [%s]",   
-                index, table_str);
-
-        ecs_os_free(table_str);
-        goto error;
+    /* If id is not a component, reflection data is missing */
+    int32_t column_index = tr->column;
+    if (column_index == -1) {
+        return flecs_json_missing_reflection(world, id, json, token, ctx, desc);
     }
 
     ecs_json_token_t token_kind = 0;
-    ecs_column_t *column = &table->data.columns[data_column];
+    ecs_column_t *column = &table->data.columns[column_index];
     ecs_type_info_t *ti = column->ti;
     ecs_size_t size = ti->size;
     ecs_entity_t type = ti->component;
-    ecs_record_t **record_array = ecs_vec_first_t(records, ecs_record_t*);
+
+    ecs_record_t **record_array = ecs_vec_first_t(&ctx->records, ecs_record_t*);
     int32_t entity = 0;
+    bool values_set = false;
+    const char *values_start = json;
 
     do {
         ecs_record_t *r = record_array[entity];
@@ -704,7 +851,14 @@ const char* flecs_json_parse_column(
 
         json = ecs_ptr_from_json(world, type, ptr, json, desc);
         if (!json) {
-            break;
+            if (desc->strict) {
+                break;
+            } else {
+                return flecs_json_missing_reflection(
+                    world, id, values_start, token, ctx, desc);
+            }
+        } else {
+            values_set = true;
         }
 
         json = flecs_json_parse(json, &token_kind, token);
@@ -718,9 +872,13 @@ const char* flecs_json_parse_column(
         entity ++;
     } while (json[0]);
 
+    if (values_set) {
+        ecs_id_t *id_set = ecs_vec_append_t(
+            ctx->a, &ctx->columns_set, ecs_id_t);
+        *id_set = id;
+    }
+
     return json;
-error:
-    return NULL;
 }
 
 static
@@ -729,17 +887,21 @@ const char* flecs_json_parse_values(
     ecs_table_t *table,
     const char *json,
     char *token,
-    ecs_vec_t *records,
-    ecs_vec_t *columns_set,
+    ecs_from_json_ctx_t *ctx,
     const ecs_from_json_desc_t *desc)
 {
-    ecs_allocator_t *a = &world->allocator;
     ecs_json_token_t token_kind = 0;
-    int32_t column = 0;
+    int32_t value = 0;
 
-    ecs_vec_clear(columns_set);
+    ecs_vec_clear(&ctx->columns_set);
 
     do {
+        if (value >= table->type.count) {
+            ecs_parser_error(desc->name, desc->expr, json - desc->expr, 
+                "more elements in values array than expected for table");
+            goto error;
+        }
+
         json = flecs_json_parse(json, &token_kind, token);
         if (!json) {
             goto error;
@@ -748,19 +910,18 @@ const char* flecs_json_parse_values(
         if (token_kind == JsonArrayClose) {
             break;
         } else if (token_kind == JsonArrayOpen) {
-            json = flecs_json_parse_column(world, table, column,
-                json, token, records, desc);
+            ecs_id_t *idptr = ecs_vec_get_t(&ctx->result_ids, ecs_id_t, value);
+            ecs_assert(idptr != NULL, ECS_INTERNAL_ERROR, NULL);
+            ecs_id_t id = idptr[0];
+            
+            json = flecs_json_parse_column(world, table, id,
+                json, token, ctx, desc);
             if (!json) {
                 goto error;
             }
-
-            ecs_id_t *id_set = ecs_vec_append_t(a, columns_set, ecs_id_t);
-            *id_set = table->type.array[column];
-
-            column ++;
         } else if (token_kind == JsonNumber) {
             if (!ecs_os_strcmp(token, "0")) {
-                column ++; /* no data */
+                /* no data */
             } else {
                 ecs_parser_error(desc->name, desc->expr, json - desc->expr, 
                     "unexpected number");
@@ -776,22 +937,25 @@ const char* flecs_json_parse_values(
                     "expected ',' or ']'");
             goto error;
         }
+
+        value ++;
     } while (json[0]);
 
     /* Send OnSet notifications */
     ecs_defer_begin(world);
     ecs_type_t type = { 
-        .array = columns_set->array, 
-        .count = columns_set->count };
+        .array = ctx->columns_set.array, 
+        .count = ctx->columns_set.count };
 
     int32_t table_count = ecs_table_count(table);
-    int32_t i, record_count = ecs_vec_count(records);
+    int32_t i, record_count = ecs_vec_count(&ctx->records);
 
     /* If the entire table was inserted, send bulk notification */
-    if (table_count == ecs_vec_count(records)) {
-        flecs_notify_on_set(world, table, 0, ecs_table_count(table), &type, true);
+    if (table_count == ecs_vec_count(&ctx->records)) {
+        flecs_notify_on_set(
+            world, table, 0, ecs_table_count(table), &type, true);
     } else {
-        ecs_record_t **rvec = ecs_vec_first_t(records, ecs_record_t*);
+        ecs_record_t **rvec = ecs_vec_first_t(&ctx->records, ecs_record_t*);
         for (i = 0; i < record_count; i ++) {
             ecs_record_t *r = rvec[i];
             int32_t row = ECS_RECORD_TO_ROW(r->row);
@@ -809,11 +973,9 @@ error:
 static
 const char* flecs_json_parse_result(
     ecs_world_t *world,
-    ecs_allocator_t *a,
     const char *json,
     char *token,
-    ecs_vec_t *records,
-    ecs_vec_t *columns_set,
+    ecs_from_json_ctx_t *ctx,
     const ecs_from_json_desc_t *desc)
 {
     ecs_json_token_t token_kind = 0;
@@ -853,11 +1015,16 @@ const char* flecs_json_parse_result(
 
     ecs_entity_t parent = 0;
     if (!ecs_os_strcmp(token, "parent")) {
-        json = flecs_json_expect(json, JsonString, token, desc);
+        char *parent_name = NULL;
+        json = flecs_json_expect_string(json, token, &parent_name, desc);
         if (!json) {
             goto error;
         }
-        parent = ecs_lookup_fullpath(world, token);
+
+        parent = flecs_json_lookup(world, 0, parent_name, desc);
+        if (parent_name != token) {
+            ecs_os_free(parent_name);
+        }
 
         json = flecs_json_expect(json, JsonComma, token, desc);
         if (!json) {
@@ -908,28 +1075,51 @@ const char* flecs_json_parse_result(
     }
 
     /* Find table from ids */
-    ecs_table_t *table = flecs_json_parse_table(world, ids, token, desc);
+    ecs_table_t *table = flecs_json_parse_table(world, ids, token, ctx, desc);
     if (!table) {
         goto error;
     }
 
     /* Add entities to table */
-    if (flecs_json_parse_entities(world, a, table, parent,
-        entities, token, records, desc)) 
+    if (flecs_json_parse_entities(world, table, parent,
+        entities, token, ctx, desc)) 
     {
         goto error;
     }
 
+    /* If not parsing in strict mode, initialize component arrays to 0. This
+     * ensures that even if components can't be deserialized (because of 
+     * incomplete reflection data) component values aren't left uninitialized */
+    if (!desc->strict) {
+        flecs_json_zeromem_table(table);
+    }
+
     /* Parse values */
     if (values) {
-        json = flecs_json_parse_values(world, table, values, token, 
-            records, columns_set, desc);
+        json = flecs_json_parse_values(world, table, values, token, ctx, desc);
         if (!json) {
             goto error;
         }
 
-        json = flecs_json_expect(json, JsonObjectClose, token, desc);
-        if (!json) {
+        json = flecs_json_parse(json, &token_kind, token);
+        if (token_kind == JsonComma) {
+            json = flecs_json_expect_member_name(json, token, "alerts", desc);
+            if (!json) {
+                goto error;
+            }
+
+            json = flecs_json_expect(json, JsonBoolean, token, desc);
+            if (!json) {
+                goto error;
+            }
+
+            json = flecs_json_expect(json, JsonObjectClose, token, desc);
+            if (!json) {
+                goto error;
+            }
+        } else if (token_kind != JsonObjectClose) {
+            ecs_parser_error(desc->name, desc->expr, json - desc->expr, 
+                "expected '}'");
             goto error;
         }
     }
@@ -949,12 +1139,8 @@ const char* ecs_world_from_json(
 
     ecs_from_json_desc_t desc = {0};
     ecs_allocator_t *a = &world->allocator;
-    ecs_vec_t records;
-    ecs_vec_t columns_set;
-    ecs_map_t anonymous_ids;
-    ecs_vec_init_t(a, &records, ecs_record_t*, 0);
-    ecs_vec_init_t(a, &columns_set, ecs_id_t, 0);
-    ecs_map_init(&anonymous_ids, a);
+    ecs_from_json_ctx_t ctx;
+    flecs_from_json_ctx_init(a, &ctx);
 
     const char *name = NULL, *expr = json, *lah;
     if (desc_arg) {
@@ -964,7 +1150,7 @@ const char* ecs_world_from_json(
     if (!desc.lookup_action) {
         desc.lookup_action = (ecs_entity_t(*)(
             const ecs_world_t*, const char*, void*))flecs_json_ensure_entity;
-        desc.lookup_ctx = &anonymous_ids;
+        desc.lookup_ctx = &ctx.anonymous_ids;
     }
 
     json = flecs_json_expect(json, JsonObjectOpen, token, &desc);
@@ -989,8 +1175,7 @@ const char* ecs_world_from_json(
     }
 
     do {
-        json = flecs_json_parse_result(world, a, json, token, 
-            &records, &columns_set, &desc);
+        json = flecs_json_parse_result(world, json, token, &ctx, &desc);
         if (!json) {
             goto error;
         }
@@ -1006,9 +1191,7 @@ const char* ecs_world_from_json(
     } while(json && json[0]);
 
 end:
-    ecs_vec_fini_t(a, &records, ecs_record_t*);
-    ecs_vec_fini_t(a, &columns_set, ecs_id_t);
-    ecs_map_fini(&anonymous_ids);
+    flecs_from_json_ctx_fini(&ctx);
 
     json = flecs_json_expect(json, JsonObjectClose, token, &desc);
     if (!json) {
@@ -1017,11 +1200,24 @@ end:
 
     return json;
 error:
-    ecs_vec_fini_t(a, &records, ecs_record_t*);
-    ecs_vec_fini_t(a, &columns_set, ecs_id_t);
-    ecs_map_fini(&anonymous_ids);
-
+    flecs_from_json_ctx_fini(&ctx);
     return NULL;
 }
 
+const char* ecs_world_from_json_file(
+    ecs_world_t *world,
+    const char *filename,
+    const ecs_from_json_desc_t *desc)
+{
+    char *json = flecs_load_from_file(filename);
+    if (!json) {
+        ecs_err("file not found: %s", filename);
+        return NULL;
+    }
+
+    const char *result = ecs_world_from_json(world, json, desc);
+    ecs_os_free(json);
+    return result;
+}
+
 #endif
