@@ -3,6 +3,9 @@
  * @brief Internal functions for JSON addon.
  */
 
+#ifndef FLECS_JSON_PRIVATE_H
+#define FLECS_JSON_PRIVATE_H
+
 #include "../../private_api.h"
 
 #ifdef FLECS_JSON
@@ -17,6 +20,7 @@ typedef enum ecs_json_token_t {
     JsonComma,
     JsonNumber,
     JsonString,
+    JsonBoolean,
     JsonTrue,
     JsonFalse,
     JsonNull,
@@ -54,6 +58,12 @@ const char* flecs_json_expect(
     char *token,
     const ecs_from_json_desc_t *desc);
 
+const char* flecs_json_expect_string(
+    const char *json,
+    char *token,
+    char **out,
+    const ecs_from_json_desc_t *desc);
+
 const char* flecs_json_expect_member(
     const char *json,
     char *token,
@@ -83,6 +93,10 @@ void flecs_json_number(
     ecs_strbuf_t *buf,
     double value);
 
+void flecs_json_u32(
+    ecs_strbuf_t *buf,
+    uint32_t value);
+
 void flecs_json_true(
     ecs_strbuf_t *buf);
 
@@ -160,8 +174,8 @@ bool flecs_json_serialize_get_field_ctx(
     ecs_json_ser_ctx_t *ser_ctx);
 
 int flecs_json_serialize_iter_result_rows(
-    const ecs_world_t *world, 
-    const ecs_iter_t *it, 
+    const ecs_world_t *world,
+    const ecs_iter_t *it,
     ecs_strbuf_t *buf,
     const ecs_iter_to_json_desc_t *desc,
     ecs_json_ser_ctx_t *ser_ctx);
@@ -173,10 +187,25 @@ bool flecs_json_serialize_iter_result_is_set(
 bool flecs_json_skip_variable(
     const char *name);
 
-int json_ser_type(
+void flecs_json_serialize_field(
     const ecs_world_t *world,
-    const ecs_vec_t *ser, 
-    const void *base, 
+    const ecs_iter_t *it,
+    const ecs_filter_t *q,
+    int field,
+    ecs_strbuf_t *buf,
+    ecs_json_ser_ctx_t *ctx);
+
+void flecs_json_serialize_query(
+    const ecs_world_t *world,
+    const ecs_filter_t *q,
+    ecs_strbuf_t *buf);
+
+int flecs_json_ser_type(
+    const ecs_world_t *world,
+    const ecs_vec_t *ser,
+    const void *base,
     ecs_strbuf_t *str);
 
 #endif
+
+#endif /* FLECS_JSON_PRIVATE_H */
