@@ -24,6 +24,9 @@ bool flecs_unit_validate(
     ecs_entity_t t,
     EcsUnit *data);
 
+void flecs_meta_import_definitions(
+    ecs_world_t *world);
+
 #endif
-    
+
 #endif
