@@ -309,7 +309,7 @@ ecs_id_t flecs_rule_op_get_id_w_written(
     if (flags_2nd & (EcsRuleIsVar | EcsRuleIsEntity)) {
         return ecs_pair(first, second);
     } else {
-        return flecs_entities_get_generation(ctx->world, first);
+        return flecs_entities_get_alive(ctx->world, first);
     }
 }
 
@@ -375,6 +375,8 @@ void flecs_rule_set_match(
     }
 
     ecs_iter_t *it = ctx->it;
+    ecs_assert(column >= 0, ECS_INTERNAL_ERROR, NULL);
+    ecs_assert(column < table->type.count, ECS_INTERNAL_ERROR, NULL);
     flecs_rule_it_set_column(it, field_index, column);
     ecs_id_t matched = flecs_rule_it_set_id(it, table, field_index, column);
     flecs_rule_set_vars(op, matched, ctx);
@@ -517,6 +519,9 @@ bool flecs_rule_with(
         op_ctx->column = flecs_ito(int16_t, tr->index);
         op_ctx->remaining = flecs_ito(int16_t, tr->count);
     } else {
+        ecs_assert((op_ctx->remaining + op_ctx->column - 1) < table->type.count, 
+            ECS_INTERNAL_ERROR, NULL);
+        ecs_assert(op_ctx->remaining >= 0, ECS_INTERNAL_ERROR, NULL);
         if (--op_ctx->remaining <= 0) {
             return false;
         }
@@ -761,6 +766,8 @@ bool flecs_rule_up_select(
                     continue;
                 }
 
+                op_ctx->row = row;
+
                 down = op_ctx->down = flecs_rule_get_down_cache(ctx, &op_ctx->cache, 
                     op_ctx->trav, entity, op_ctx->idr_with, self);
                 op_ctx->cache_elem = -1;
@@ -836,7 +843,7 @@ bool flecs_rule_up_with(
             return false;
         }
 
-        it->sources[op->field_index] = flecs_entities_get_generation(
+        it->sources[op->field_index] = flecs_entities_get_alive(
             ctx->world, up->src);
         it->columns[op->field_index] = up->column + 1;
         it->ids[op->field_index] = up->id;
@@ -859,6 +866,8 @@ bool flecs_rule_self_up_with(
         bool result;
         if (id_only) {
             result = flecs_rule_with_id(op, redo, ctx);
+            ecs_rule_and_ctx_t *op_ctx = flecs_op_ctx(ctx, and);
+            op_ctx->remaining = 1;
         } else {
             result = flecs_rule_with(op, redo, ctx);
         }
@@ -1666,7 +1675,7 @@ bool flecs_rule_pred_eq_name(
     ecs_rule_run_ctx_t *ctx)
 {
     const char *name = flecs_rule_name_arg(op, ctx);
-    ecs_entity_t e = ecs_lookup_fullpath(ctx->world, name);
+    ecs_entity_t e = ecs_lookup(ctx->world, name);
     if (!e) {
         /* Entity doesn't exist */
         return false;
@@ -1872,7 +1881,7 @@ bool flecs_rule_pred_neq_name(
     ecs_rule_run_ctx_t *ctx)
 {
     const char *name = flecs_rule_name_arg(op, ctx);
-    ecs_entity_t e = ecs_lookup_fullpath(ctx->world, name);
+    ecs_entity_t e = ecs_lookup(ctx->world, name);
     if (!e) {
         /* Entity doesn't exist */
         return true && !redo;
@@ -2325,8 +2334,11 @@ bool flecs_rule_populate(
                 ecs_record_t *r = flecs_entities_get(ctx->world, src);
                 ecs_table_t *src_table = r->table;
                 if (src_table->column_map) {
+                    ecs_assert(index <= src_table->type.count, 
+                        ECS_INTERNAL_ERROR, NULL);
                     int32_t column = src_table->column_map[index - 1];
                     if (column != -1) {
+                        ecs_assert(column >= 0, ECS_INTERNAL_ERROR, NULL);
                         it->ptrs[i] = ecs_vec_get(
                             &src_table->data.columns[column].data,
                             it->sizes[i],
@@ -2487,8 +2499,6 @@ bool flecs_rule_run_until(
             return true;
         }
     } while (true);
-
-    return false;
 }
 
 static
@@ -2741,6 +2751,9 @@ ecs_iter_t ecs_rule_iter(
     ecs_rule_iter_t *rit = &it.priv.iter.rule;
     ecs_check(rule != NULL, ECS_INVALID_PARAMETER, NULL);
 
+    // Ok, only for stats
+    ECS_CONST_CAST(ecs_rule_t*, rule)->filter.eval_count ++;
+
     ecs_run_aperiodic(rule->filter.world, EcsAperiodicEmptyTables);
 
     int32_t i, var_count = rule->var_count, op_count = rule->op_count;
