@@ -104,6 +104,7 @@ ecs_entity_t ecs_run_intern(
         it = &wit;
     }
 
+    ecs_entity_t old_system = flecs_stage_set_system(stage, system);
     ecs_iter_action_t action = system_data->action;
     it->callback = action;
     
@@ -125,12 +126,12 @@ ecs_entity_t ecs_run_intern(
         ecs_iter_fini(&qit);
     }
 
+    flecs_stage_set_system(stage, old_system);
+
     if (measure_time) {
         system_data->time_spent += (ecs_ftime_t)ecs_time_measure(&time_start);
     }
 
-    system_data->invoke_count ++;
-
     flecs_defer_end(world, stage);
 
     return it->interrupted_by;
@@ -229,6 +230,13 @@ void flecs_system_fini(ecs_system_t *sys) {
     ecs_poly_free(sys, ecs_system_t);
 }
 
+/* ecs_poly_dtor_t-compatible wrapper */
+static
+void flecs_system_fini_poly(void *sys)
+{
+    flecs_system_fini(sys);
+}
+
 static
 void flecs_system_init_timer(
     ecs_world_t *world,
@@ -281,7 +289,7 @@ ecs_entity_t ecs_system_init(
         
         poly->poly = system;
         system->world = world;
-        system->dtor = (ecs_poly_dtor_t)flecs_system_fini;
+        system->dtor = flecs_system_fini_poly;
         system->entity = entity;
 
         ecs_query_desc_t query_desc = desc->query;
@@ -379,7 +387,7 @@ void FlecsSystemImport(
     ecs_world_t *world)
 {
     ECS_MODULE(world, FlecsSystem);
-#ifdef FLECS_COREDOC
+#ifdef FLECS_DOC
     ECS_IMPORT(world, FlecsDoc);
     ecs_doc_set_brief(world, ecs_id(FlecsSystem), 
         "Module that implements Flecs systems");
