@@ -33,12 +33,16 @@ void flecs_bootstrap(
     });
 
 #define flecs_bootstrap_tag(world, name)\
-    ecs_ensure(world, name);\
+    ecs_make_alive(world, name);\
     ecs_add_id(world, name, EcsFinal);\
     ecs_add_pair(world, name, EcsChildOf, ecs_get_scope(world));\
-    ecs_set(world, name, EcsComponent, {.size = 0});\
     ecs_set_name(world, name, (const char*)&#name[ecs_os_strlen(world->info.name_prefix)]);\
-    ecs_set_symbol(world, name, #name)
+    ecs_set_symbol(world, name, #name);
+
+#define flecs_bootstrap_trait(world, name)\
+    flecs_bootstrap_tag(world, name)\
+    ecs_add_id(world, name, EcsTrait)
+
 
 /* Bootstrap functions for other parts in the code */
 void flecs_bootstrap_hierarchy(ecs_world_t *world);
@@ -268,13 +272,22 @@ const char* flecs_name_from_symbol(
     ecs_world_t *world,
     const char *type_name);
 
-/* Compare function for entity ids */
+/* Compare function for entity ids used for order_by */
 int flecs_entity_compare(
     ecs_entity_t e1, 
     const void *ptr1, 
     ecs_entity_t e2, 
     const void *ptr2); 
 
+/* Compare function for component ids used for qsort */
+int flecs_id_qsort_cmp(
+    const void *a, 
+    const void *b);
+
+/* Load file contents into string */
+char* flecs_load_from_file(
+    const char *filename);
+
 bool flecs_name_is_id(
     const char *name);
 
