@@ -883,7 +883,7 @@ ecs_query_table_match_t* flecs_query_add_table_match(
     qm->sources = flecs_balloc(&query->allocators.sources);
 
     /* Insert match to iteration list if table is not empty */
-    if (!table || ecs_table_count(table) != 0) {
+    if (!table || ecs_table_count(table) != 0 || (query->flags & EcsQueryMatchEmptyTables)) {
         flecs_query_insert_table_node(query, qm);
     }
 
@@ -981,7 +981,12 @@ ecs_query_table_t* flecs_query_table_insert(
     } else {
         qt->table_id = 0;
     }
-    ecs_table_cache_insert(&query->cache, table, &qt->hdr);
+
+    if (query->flags & EcsQueryMatchEmptyTables) {
+        ecs_table_cache_insert_w_empty(&query->cache, table, &qt->hdr, false);
+    } else {
+        ecs_table_cache_insert(&query->cache, table, &qt->hdr);
+    }
     return qt;
 }
 
@@ -994,7 +999,7 @@ void flecs_query_match_tables(
     ecs_table_t *table = NULL;
     ecs_query_table_t *qt = NULL;
 
-    ecs_iter_t it = ecs_filter_iter(world, &query->filter);
+    ecs_iter_t it = flecs_filter_iter_w_flags(world, &query->filter, 0);
     ECS_BIT_SET(it.flags, EcsIterIsInstanced);
     ECS_BIT_SET(it.flags, EcsIterNoData);
     ECS_BIT_SET(it.flags, EcsIterTableOnly);
@@ -1150,7 +1155,9 @@ void flecs_query_build_sorted_table_range(
         ecs_table_t *table = cur->table;
         ecs_data_t *data = &table->data;
 
-        ecs_assert(ecs_table_count(table) != 0, ECS_INTERNAL_ERROR, NULL);
+        if (ecs_table_count(table) == 0) {
+            continue;
+        }
 
         if (id) {
             const ecs_term_t *term = &query->filter.terms[order_by_term];
@@ -1304,7 +1311,6 @@ void flecs_query_sort_tables(
     }
 
     ecs_sort_table_action_t sort = query->sort_table;
-    
     ecs_entity_t order_by_component = query->order_by_component;
     int32_t order_by_term = query->order_by_term;
 
@@ -1323,6 +1329,7 @@ void flecs_query_sort_tables(
         bool dirty = false;
 
         if (flecs_query_check_table_monitor(query, qt, 0)) {
+            tables_sorted = true; /* Ensure ranges are rebuilt */
             dirty = true;
         }
 
@@ -1639,7 +1646,7 @@ void flecs_query_rematch_tables(
         parent_it = ecs_query_iter(world, parent_query);
         it = ecs_filter_chain_iter(&parent_it, &query->filter);
     } else {
-        it = ecs_filter_iter(world, &query->filter);
+        it = flecs_filter_iter_w_flags(world, &query->filter, 0);
     }
 
     ECS_BIT_SET(it.flags, EcsIterIsInstanced);
@@ -2027,6 +2034,13 @@ void flecs_query_fini(
     ecs_poly_free(query, ecs_query_t);
 }
 
+/* ecs_poly_dtor_t-compatible wrapper */
+static
+void flecs_query_fini_poly(void *query)
+{
+    flecs_query_fini(query);
+}
+
 /* -- Public API -- */
 
 ecs_query_t* ecs_query_init(
@@ -2041,8 +2055,9 @@ ecs_query_t* ecs_query_init(
     ecs_query_t *result = ecs_poly_new(ecs_query_t);
     ecs_observer_desc_t observer_desc = { .filter = desc->filter };
     ecs_entity_t entity = desc->filter.entity;
+    ecs_flags32_t filter_flags = desc->filter.flags|world->default_query_flags;
 
-    observer_desc.filter.flags = EcsFilterMatchEmptyTables;
+    observer_desc.filter.flags = EcsFilterMatchEmptyTables|filter_flags;
     observer_desc.filter.storage = &result->filter;
     result->filter = ECS_FILTER_INIT;
 
@@ -2052,6 +2067,8 @@ ecs_query_t* ecs_query_init(
 
     ECS_BIT_COND(result->flags, EcsQueryTrivialIter, 
         !!(result->filter.flags & EcsFilterMatchOnlyThis));
+    ECS_BIT_COND(result->flags, EcsQueryMatchEmptyTables, 
+        !!(filter_flags & EcsFilterMatchEmptyTables));
 
     flecs_query_allocators_init(result);
 
@@ -2059,29 +2076,36 @@ ecs_query_t* ecs_query_init(
         observer_desc.entity = entity;
         observer_desc.run = flecs_query_on_event;
         observer_desc.ctx = result;
-        observer_desc.events[0] = EcsOnTableEmpty;
-        observer_desc.events[1] = EcsOnTableFill;
+
+        int32_t event_index = 0;
+        if (!(filter_flags & EcsFilterMatchEmptyTables)) {
+            observer_desc.events[event_index ++] = EcsOnTableEmpty;
+            observer_desc.events[event_index ++] = EcsOnTableFill;
+        }
         if (!desc->parent) {
-            observer_desc.events[2] = EcsOnTableCreate;
-            observer_desc.events[3] = EcsOnTableDelete;
+            observer_desc.events[event_index ++] = EcsOnTableCreate;
+            observer_desc.events[event_index ++] = EcsOnTableDelete;
         }
-        observer_desc.filter.flags |= EcsFilterNoData;
-        observer_desc.filter.instanced = true;
 
-        /* ecs_filter_init could have moved away resources from the terms array
-         * in the descriptor, so use the terms array from the filter. */
-        observer_desc.filter.terms_buffer = result->filter.terms;
-        observer_desc.filter.terms_buffer_count = result->filter.term_count;
-        observer_desc.filter.expr = NULL; /* Already parsed */
+        if (event_index) {
+            observer_desc.filter.flags |= EcsFilterNoData;
+            observer_desc.filter.instanced = true;
 
-        entity = ecs_observer_init(world, &observer_desc);
-        if (!entity) {
-            goto error;
+            /* ecs_filter_init could have moved away resources from the terms array
+            * in the descriptor, so use the terms array from the filter. */
+            observer_desc.filter.terms_buffer = result->filter.terms;
+            observer_desc.filter.terms_buffer_count = result->filter.term_count;
+            observer_desc.filter.expr = NULL; /* Already parsed */
+
+            entity = ecs_observer_init(world, &observer_desc);
+            if (!entity) {
+                goto error;
+            }
         }
     }
 
     result->iterable.init = flecs_query_iter_init;
-    result->dtor = (ecs_poly_dtor_t)flecs_query_fini;
+    result->dtor = flecs_query_fini_poly;
     result->prev_match_count = -1;
 
     result->ctx = desc->ctx;
@@ -2246,9 +2270,13 @@ ecs_iter_t ecs_query_iter(
     ecs_world_t *world = query->filter.world;
     ecs_poly_assert(world, ecs_world_t);
 
+    query->filter.eval_count ++;
+
     /* Process table events to ensure that the list of iterated tables doesn't
-     * contain empty tables. */
-    flecs_process_pending_tables(world);
+     * contain empty tables. Not necessary if query matches empty tables. */
+    if (!(query->flags & EcsQueryMatchEmptyTables)) {
+        flecs_process_pending_tables(world);
+    }
 
     /* If query has order_by, apply sort */
     flecs_query_sort_tables(world, query);
@@ -2499,30 +2527,33 @@ void flecs_query_populate_trivial(
     it->references = ecs_vec_first(&match->refs);
 
     if (!it->references) {
-        ecs_data_t *data = &table->data;
-        if (!(it->flags & EcsIterNoData)) {
-            int32_t i;
-            for (i = 0; i < it->field_count; i ++) {
-                int32_t column = match->storage_columns[i];
-                if (column < 0) {
-                    it->ptrs[i] = NULL;
-                    continue;
-                }
+        if (count) {
+            ecs_data_t *data = &table->data;
+            if (!(it->flags & EcsIterNoData)) {
+                int32_t i;
+                for (i = 0; i < it->field_count; i ++) {
+                    int32_t column = match->storage_columns[i];
+                    if (column < 0) {
+                        it->ptrs[i] = NULL;
+                        continue;
+                    }
 
-                ecs_size_t size = it->sizes[i];
-                if (!size) {
-                    it->ptrs[i] = NULL;
-                    continue;
-                }
+                    ecs_size_t size = it->sizes[i];
+                    if (!size) {
+                        it->ptrs[i] = NULL;
+                        continue;
+                    }
 
-                it->ptrs[i] = ecs_vec_get(&data->columns[column].data,
-                    it->sizes[i], offset);
+                    it->ptrs[i] = ecs_vec_get(&data->columns[column].data,
+                        it->sizes[i], offset);
+                }
             }
+
+            it->entities = ecs_vec_get_t(&data->entities, ecs_entity_t, offset);
         }
 
         it->frame_offset += it->table ? ecs_table_count(it->table) : 0;
         it->table = table;
-        it->entities = ecs_vec_get_t(&data->entities, ecs_entity_t, offset);
     } else {
         flecs_iter_populate_data(
             it->real_world, it, table, offset, count, it->ptrs);
@@ -2578,7 +2609,6 @@ repeat:
         range->count = match->count;
         if (!range->count) {
             range->count = ecs_table_count(table);
-            ecs_assert(range->count != 0, ECS_INTERNAL_ERROR, NULL);
         }
 
         if (match->entity_filter) {
