@@ -497,7 +497,7 @@ void ComponentLifecycle_struct_w_vector_set_2_remove_w_tag(void) {
     test_assert(ptr2 == NULL);
 }
 
-void ComponentLifecycle_get_mut_new(void) {
+void ComponentLifecycle_ensure_new(void) {
     flecs::world world;
 
     world.component<Pod>();
@@ -505,8 +505,7 @@ void ComponentLifecycle_get_mut_new(void) {
     auto e = world.entity();
     test_assert(e.id() != 0);
 
-    Pod* value = e.get_mut<Pod>();
-    test_assert(value != NULL);
+    e.ensure<Pod>();
 
     Pod::ctor_invoked = 1;
     Pod::dtor_invoked = 0;
@@ -521,7 +520,7 @@ void ComponentLifecycle_get_mut_new(void) {
     Pod::move_invoked = 0;    
 }
 
-void ComponentLifecycle_get_mut_existing(void) {
+void ComponentLifecycle_ensure_existing(void) {
     flecs::world world;
 
     world.component<Pod>();
@@ -529,18 +528,16 @@ void ComponentLifecycle_get_mut_existing(void) {
     auto e = world.entity();
     test_assert(e.id() != 0);
 
-    Pod* value = e.get_mut<Pod>();
-    test_assert(value != NULL);
+    e.ensure<Pod>();
 
     Pod::ctor_invoked = 1;
     Pod::dtor_invoked = 0;
     Pod::copy_invoked = 0;
     Pod::move_invoked = 0;
 
-    value = e.get_mut<Pod>();
-    test_assert(value != NULL);
+    e.ensure<Pod>();
 
-    /* Repeated calls to get_mut should not invoke constructor */
+    /* Repeated calls to ensure should not invoke constructor */
     Pod::ctor_invoked = 1;
     Pod::dtor_invoked = 0;
     Pod::copy_invoked = 0;
@@ -1783,10 +1780,10 @@ void ComponentLifecycle_dtor_with_relation(void) {
         test_int(Pod::move_ctor_invoked, 1);
     }
 
-    test_int(Pod::ctor_invoked, 5);
+    test_int(Pod::ctor_invoked, 4);
     test_int(Pod::dtor_invoked, 6);
-    test_int(Pod::move_invoked, 4);
-    test_int(Pod::move_ctor_invoked, 1);
+    test_int(Pod::move_invoked, 3);
+    test_int(Pod::move_ctor_invoked, 2);
 }
 
 void ComponentLifecycle_register_parent_after_child_w_hooks(void) {
@@ -1821,3 +1818,39 @@ void ComponentLifecycle_register_parent_after_child_w_hooks_implicit(void) {
     test_int(Pod::copy_invoked, 0);
     test_int(Pod::copy_ctor_invoked, 0);
 }
+
+void ComponentLifecycle_dtor_relation_target(void) {
+    {
+        flecs::world ecs;
+
+        auto e = ecs.entity();
+        auto e2 = ecs.entity().emplace<CountNoDefaultCtor>(5).add<Tag>(e);
+        ecs.entity().emplace<CountNoDefaultCtor>(5);
+
+        test_int(CountNoDefaultCtor::ctor_invoked, 2);
+        test_int(CountNoDefaultCtor::dtor_invoked, 1);
+        test_int(CountNoDefaultCtor::move_invoked, 0);
+        test_int(CountNoDefaultCtor::move_ctor_invoked, 1);
+
+        const CountNoDefaultCtor *ptr = e2.get<CountNoDefaultCtor>();
+        test_assert(ptr != NULL);
+        test_int(ptr->value, 5);
+
+        test_int(CountNoDefaultCtor::ctor_invoked, 2);
+        test_int(CountNoDefaultCtor::dtor_invoked, 1);
+        test_int(CountNoDefaultCtor::move_invoked, 0);
+        test_int(CountNoDefaultCtor::move_ctor_invoked, 1);
+
+        e.destruct();
+
+        test_int(CountNoDefaultCtor::ctor_invoked, 2);
+        test_int(CountNoDefaultCtor::dtor_invoked, 2);
+        test_int(CountNoDefaultCtor::move_invoked, 0);
+        test_int(CountNoDefaultCtor::move_ctor_invoked, 2);
+    }
+
+    test_int(CountNoDefaultCtor::ctor_invoked, 2);
+    test_int(CountNoDefaultCtor::dtor_invoked, 4);
+    test_int(CountNoDefaultCtor::move_invoked, 0);
+    test_int(CountNoDefaultCtor::move_ctor_invoked, 2);
+}
