@@ -147,7 +147,7 @@ void Pairs_system_1_pair_instance(void) {
     ecs.system<>()
         .expr("(Pair, *)")
         .iter([&](flecs::iter it) {
-            flecs::column<Pair> tr(it, 1);
+            flecs::field<Pair> tr(it, 1);
             invoke_count ++;            
             for (auto i : it) {
                 entity_count ++;
@@ -176,7 +176,7 @@ void Pairs_system_2_pair_instances(void) {
     ecs.system<>()
         .expr("(Pair, *)")
         .iter([&](flecs::iter it) {
-            flecs::column<Pair> tr(it, 1);
+            flecs::field<Pair> tr(it, 1);
             invoke_count ++;            
             for (auto i : it) {
                 entity_count ++;
@@ -250,55 +250,53 @@ void Pairs_override_tag_pair(void) {
     test_assert(t == t_2); 
 }
 
-void Pairs_get_mut_pair(void) {
+void Pairs_ensure_pair(void) {
     flecs::world ecs;
 
     auto e = ecs.entity();
 
-    Pair *t = e.get_mut<Pair, Position>();
-    test_assert(t != NULL);
-    t->value = 10;
+    Pair& t = e.ensure<Pair, Position>();
+    t.value = 10;
 
     const Pair *t_2 = e.get<Pair, Position>();
-    test_assert(t == t_2);
-    test_int(t->value, 10);
+    test_int(t.value, 10);
+    test_int(t_2->value, 10);
 }
 
-void Pairs_get_mut_pair_existing(void) {
+void Pairs_ensure_pair_existing(void) {
     flecs::world ecs;
 
     auto e = ecs.entity()
         .set<Pair, Position>({20});
 
-    Pair *t = e.get_mut<Pair, Position>();
-    test_assert(t != NULL);
-    test_int(t->value, 20);
-    t->value = 10;
+    Pair& t = e.ensure<Pair, Position>();
+    test_int(t.value, 20);
+    t.value = 10;
 
     const Pair *t_2 = e.get<Pair, Position>();
-    test_assert(t == t_2);
-    test_int(t->value, 10);
+    test_int(t.value, 10);
+    test_int(t_2->value, 10);
 }
 
-void Pairs_get_mut_pair_tag(void) {
+void Pairs_ensure_pair_tag(void) {
     flecs::world ecs;
 
     auto Pair = ecs.entity();
 
     auto e = ecs.entity();
 
-    Position *p = e.get_mut_second<Position>(Pair);
-    test_assert(p != NULL);
-    p->x = 10;
-    p->y = 20;
+    Position& p = e.ensure_second<Position>(Pair);
+    p.x = 10;
+    p.y = 20;
 
     const Position *p_2 = e.get_second<Position>(Pair);
-    test_assert(p == p_2);
-    test_int(p->x, 10);
-    test_int(p->y, 20);
+    test_int(p.x, 10);
+    test_int(p.y, 20);
+    test_int(p_2->x, 10);
+    test_int(p_2->y, 20);
 }
 
-void Pairs_get_mut_pair_tag_existing(void) {
+void Pairs_ensure_pair_tag_existing(void) {
     flecs::world ecs;
 
     auto Pair = ecs.entity();
@@ -306,34 +304,34 @@ void Pairs_get_mut_pair_tag_existing(void) {
     auto e = ecs.entity()
         .set_second<Position>(Pair, {10, 20});
 
-    Position *p = e.get_mut_second<Position>(Pair);
-    test_assert(p != NULL);
-    test_int(p->x, 10);
-    test_int(p->y, 20);
+    Position& p = e.ensure_second<Position>(Pair);
+    test_int(p.x, 10);
+    test_int(p.y, 20);
 
     const Position *p_2 = e.get_second<Position>(Pair);
-    test_assert(p == p_2);
-    test_int(p->x, 10);
-    test_int(p->y, 20);
+    test_int(p.x, 10);
+    test_int(p.y, 20);
+    test_int(p_2->x, 10);
+    test_int(p_2->y, 20);
 }
 
-void Pairs_get_mut_R_tag_O(void) {
+void Pairs_ensure_R_tag_O(void) {
     flecs::world ecs;
 
     auto e = ecs.entity()
         .set<Tag, Position>({10, 20});
 
-    Position *t = e.get_mut<Tag, Position>();
-    test_assert(t != NULL);
-    test_int(t->x, 10);
-    test_int(t->y, 20);
-    t->x = 30;
-    t->y = 40;
+    Position& t = e.ensure<Tag, Position>();
+    test_int(t.x, 10);
+    test_int(t.y, 20);
+    t.x = 30;
+    t.y = 40;
 
     const Position *t_2 = e.get<Tag, Position>();
-    test_assert(t == t_2);
-    test_int(t->x, 30);
-    test_int(t->y, 40);
+    test_int(t.x, 30);
+    test_int(t.y, 40);
+    test_int(t_2->x, 30);
+    test_int(t_2->y, 40);
 }
 
 void Pairs_get_relation_from_id(void) {
