@@ -279,8 +279,8 @@ void System_signature(void) {
 
     world.system<>().expr("Position, Velocity")
         .iter([](flecs::iter&it) {
-            flecs::column<Position> p(it, 1);
-            flecs::column<Velocity> v(it, 2);
+            flecs::field<Position> p(it, 1);
+            flecs::field<Velocity> v(it, 2);
 
             for (auto i : it) {
                 p[i].x += v[i].x;
@@ -308,8 +308,8 @@ void System_signature_const(void) {
 
     world.system<>().expr("Position, [in] Velocity")
         .iter([](flecs::iter&it) {
-            flecs::column<Position> p(it, 1);
-            flecs::column<const Velocity> v(it, 2);
+            flecs::field<Position> p(it, 1);
+            flecs::field<const Velocity> v(it, 2);
 
             for (auto i : it) {
                 p[i].x += v[i].x;
@@ -344,8 +344,8 @@ void System_signature_shared(void) {
 
     world.system<>().expr("Position, [in] Velocity(self|up)")
         .iter([](flecs::iter&it) {
-            flecs::column<Position> p(it, 1);
-            flecs::column<const Velocity> v(it, 2);
+            flecs::field<Position> p(it, 1);
+            flecs::field<const Velocity> v(it, 2);
 
             if (!it.is_self(2)) {
                 for (auto i : it) {
@@ -393,9 +393,9 @@ void System_signature_optional(void) {
 
     world.system<>().expr("Position, ?Velocity, ?Mass")
         .iter([](flecs::iter& it) {
-            flecs::column<Position> p(it, 1);
-            flecs::column<Velocity> v(it, 2);
-            flecs::column<Mass> m(it, 3);
+            flecs::field<Position> p(it, 1);
+            flecs::field<Velocity> v(it, 2);
+            flecs::field<Mass> m(it, 3);
 
             if (it.is_set(2) && it.is_set(3)) {
                 for (auto i : it) {
@@ -2043,8 +2043,8 @@ void System_interval_tick_source(void) {
 
     flecs::timer t = ecs.timer().interval(2.1);
 
-    flecs::Timer *timer = t.get_mut<flecs::Timer>();
-    timer->time = 0;
+    flecs::Timer& timer = t.ensure<flecs::Timer>();
+    timer.time = 0;
 
     int32_t sys_a_invoked = 0, sys_b_invoked = 0;
 
@@ -2287,3 +2287,36 @@ void System_singleton_tick_source(void) {
     ecs.progress(2.0);
     test_int(1, sys_invoked);
 }
+
+enum class PipelineStepEnum
+{
+    CustomStep,
+    CustomStep2
+};
+
+void System_pipeline_step_with_kind_enum(void) {
+    flecs::world ecs;
+
+    ecs.entity(PipelineStepEnum::CustomStep).add(flecs::Phase).depends_on(flecs::OnStart);
+
+    bool ran_test = false;
+
+    ecs.system().kind(PipelineStepEnum::CustomStep).iter([&ran_test](flecs::iter& it) { ran_test = true; });
+
+    ecs.progress();
+    test_assert(ran_test);
+}
+
+void System_pipeline_step_depends_on_pipeline_step_with_enum(void) {
+    flecs::world ecs;
+
+    ecs.entity(PipelineStepEnum::CustomStep).add(flecs::Phase).depends_on(flecs::OnStart);
+    ecs.entity(PipelineStepEnum::CustomStep2).add(flecs::Phase).depends_on(PipelineStepEnum::CustomStep);
+
+    bool ran_test = false;
+
+    ecs.system().kind(PipelineStepEnum::CustomStep2).iter([&ran_test](flecs::iter& it) { ran_test = true; });
+
+    ecs.progress();
+    test_assert(ran_test);
+}
