@@ -92,7 +92,7 @@ void World_builtin_components(void) {
     test_assert(ecs.component<flecs::Component>() == ecs_id(EcsComponent));
     test_assert(ecs.component<flecs::Identifier>() == ecs_id(EcsIdentifier));
     test_assert(ecs.component<flecs::Poly>() == ecs_id(EcsPoly));
-    test_assert(ecs.component<flecs::Target>() == ecs_id(EcsTarget));
+    test_assert(ecs.component<flecs::FlattenTarget>() == ecs_id(EcsFlattenTarget));
     test_assert(ecs.component<flecs::RateFilter>() == ecs_id(EcsRateFilter));
     test_assert(ecs.component<flecs::TickSource>() == ecs_id(EcsTickSource));
     test_assert(flecs::Name == EcsName);
@@ -1512,7 +1512,7 @@ void World_get_alive(void) {
     test_assert(ecs.get_alive(e_no_gen) == e_2);
 }
 
-void World_ensure(void) {
+void World_make_alive(void) {
     flecs::world ecs;
 
     auto e_1 = ecs.entity();
@@ -1525,7 +1525,7 @@ void World_ensure(void) {
     e_2.destruct();
     test_assert(!e_2.is_alive());
 
-    auto e_3 = ecs.ensure(e_2);
+    auto e_3 = ecs.make_alive(e_2);
     test_assert(e_2 == e_3);
     test_assert(e_3.is_alive());
 }
@@ -1867,3 +1867,71 @@ void World_register_nested_component_in_module(void) {
     test_str(foo.path().c_str(), "::nested_component_module::Foo");
     test_str(bar.path().c_str(), "::nested_component_module::Foo::Bar");
 }
+
+static void *atfini_ctx = nullptr;
+static int atfini_invoked = 0;
+static void atfini_callback(flecs::world_t *world, void *ctx) {
+    test_assert(world != nullptr);
+    atfini_ctx = ctx;
+    atfini_invoked ++;
+}
+
+void World_atfini(void) {
+    {
+        flecs::world ecs;
+        ecs.atfini(atfini_callback);
+    }
+    test_int(atfini_invoked, 1);
+    test_assert(atfini_ctx == nullptr);
+}
+
+void World_atfini_w_ctx(void) {
+    int ctx;
+    {
+        flecs::world ecs;
+        ecs.atfini(atfini_callback, &ctx);
+    }
+    test_int(atfini_invoked, 1);
+    test_assert(atfini_ctx == &ctx);
+}
+
+void World_get_mut_T(void) {
+    flecs::world world;
+
+    Position *p = world.get_mut<Position>();
+
+    test_assert(p == nullptr);
+
+    world.set<Position>({10, 20});
+    p = world.get_mut<Position>();
+
+    test_assert(p != nullptr);
+    test_int(p->x, 10);
+    test_int(p->y, 20);
+}
+
+void World_get_mut_R_T(void) {
+    flecs::world world;
+
+    struct Tgt { };
+
+    Position *p = world.get_mut<Position, Tgt>();
+    test_assert(p == nullptr);
+
+    world.set<Position, Tgt>({10, 20});
+    
+    p = world.get_mut<Position, Tgt>();
+    test_assert(p != nullptr);
+
+    test_int(p->x, 10);
+    test_int(p->y, 20);
+}
+
+void World_world_mini(void) {
+    flecs::world world(ecs_mini(), true);
+
+    test_assert(world.lookup("flecs.system") == 0);
+    test_assert(world.lookup("flecs.pipeline") == 0);
+    test_assert(world.lookup("flecs.timer") == 0);
+    test_assert(world.lookup("flecs.meta") == 0);
+}
